package de.ozgcloud.common.grpc;

import static org.assertj.core.api.Assertions.*;

import com.thedeanda.lorem.LoremIpsum;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import io.grpc.Metadata;

import java.util.UUID;

class GrpcUtilTest {

	private static final String KEY_VALUE_BIN = "key_value-bin";
	private static final String KEY_VALUE_STRING = "key_value";
	private static final String UNKNOWN_KEY_BIN = "unknown_key-bin";

	private final Metadata header = new Metadata();

	@DisplayName("Test creation of MetaData keys")
	@Nested
	class TestKeyGeneration {

		@Test
		void shouldCreateBinaryKey() {
			var key = GrpcUtil.createKeyOf(KEY_VALUE_BIN);

			assertThat(key.name()).isEqualTo(KEY_VALUE_BIN);
		}

		@Test
		void shouldThrowExceptionBecauseOfMissingBinSuffix() {
			assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> GrpcUtil.createKeyOf("key"));
		}

		@Test
		void shouldCreateAsciiKey() {
			var key = GrpcUtil.keyOfString(KEY_VALUE_STRING);

			assertThat(key.name()).isEqualTo(KEY_VALUE_STRING);
		}

		@Test
		void shouldThrowExceptionBecauseOfBinSuffix() {
			assertThatExceptionOfType(IllegalArgumentException.class).isThrownBy(() -> GrpcUtil.keyOfString(KEY_VALUE_BIN));
		}
	}

	@DisplayName("Test reading values from the grpc header")
	@Nested
	class TestReadingFromHeader {
		private static final String TEST_VALUE = LoremIpsum.getInstance().getWords(1);

		@BeforeEach
		void initMetadata() {
			header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE.getBytes());
		}

		@Test
		void shouldReadFromHeader() {
			var value = GrpcUtil.getFromHeaders(KEY_VALUE_BIN, header);

			assertThat(value).contains(TEST_VALUE);
		}

		@Test
		void shouldReturnNullOnUnknownKey() {
			var value = GrpcUtil.getFromHeaders(UNKNOWN_KEY_BIN, header);

			assertThat(value).isEmpty();
		}
	}

	@DisplayName("Test reading multiple values of a key from the grpc header")
	@Nested
	class TestReadingCollection {
		private static final String TEST_VALUE_1 = LoremIpsum.getInstance().getWords(1);
		private static final String TEST_VALUE_2 = LoremIpsum.getInstance().getWords(2);

		@BeforeEach
		void initMetadata() {
			header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE_1.getBytes());
			header.put(GrpcUtil.createKeyOf(KEY_VALUE_BIN), TEST_VALUE_2.getBytes());
		}

		@Test
		void shouldReadFromHeader() {
			var values = GrpcUtil.getList(KEY_VALUE_BIN, header);

			assertThat(values).hasSize(2).contains(TEST_VALUE_1, TEST_VALUE_2);
		}

		@Test
		void shouldGetEmptyCollectionOnUnknownKey() {
			var values = GrpcUtil.getList(UNKNOWN_KEY_BIN, header);

			assertThat(values).isEmpty();
		}
	}

	@Nested
	class TestGetRequestId {

		private static final String REQUEST_ID = UUID.randomUUID().toString();

		@Test
		void shouldReturnRequestId() {
			header.put(GrpcUtil.HEADER_KEY_REQUEST_ID, REQUEST_ID.getBytes());

			var result = GrpcUtil.getRequestId(header);

			assertThat(result).contains(REQUEST_ID);
		}
	}

	@Nested
	class TestGetClientName {

		private static final String CLIENT_NAME = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldReturnClientName() {
			header.put(GrpcUtil.HEADER_KEY_CLIENT_NAME, CLIENT_NAME.getBytes());

			var result = GrpcUtil.getClientName(header);

			assertThat(result).contains(CLIENT_NAME);
		}
	}

	@Nested
	class TestGetUserId {

		private static final String USER_ID = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldReturnUserId() {
			header.put(GrpcUtil.HEADER_KEY_USER_ID, USER_ID.getBytes());

			var result = GrpcUtil.getUserId(header);

			assertThat(result).contains(USER_ID);
		}
	}

	@Nested
	class TestGetUserName {

		private static final String USER_NAME = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldReturnUserName() {
			header.put(GrpcUtil.HEADER_KEY_USER_NAME, USER_NAME.getBytes());

			var result = GrpcUtil.getUserName(header);

			assertThat(result).contains(USER_NAME);
		}
	}

	@Nested
	class TestGetAccessLimitedOrgaId {

		private static final String ACCESS_LIMITED_ORGAID = LoremIpsum.getInstance().getWords(1);

		@Test
		void shouldReturnAccessLimitedOrgaId() {
			header.put(GrpcUtil.HEADER_KEY_ACCESS_LIMITED_ORGAID, ACCESS_LIMITED_ORGAID.getBytes());

			var result = GrpcUtil.getAccessLimitedOrgaIds(header);

			assertThat(result).containsExactly(ACCESS_LIMITED_ORGAID);
		}
	}

	@Nested
	class TestGetAccessLimited {

		private static final boolean ACCESS_LIMITED = true;

		@Test
		void shouldReturnAccessLimited() {
			header.put(GrpcUtil.HEADER_KEY_ACCESS_LIMITED, String.valueOf(ACCESS_LIMITED).getBytes());

			var result = GrpcUtil.getAccessLimited(header);

			assertThat(result).contains(ACCESS_LIMITED);
		}
	}
}
